@@ -3,21 +3,22 @@
 
 #include "common.h"
 
-#include <stdbool.h>
+#include <stddef.h>
 #include <stdint.h>
-#include <unistd.h>
+#include <sys/types.h>
 
 #define DEVICE_MSG_MAX_SIZE (1 << 18) // 256k
 // type: 1 byte; length: 4 bytes
 #define DEVICE_MSG_TEXT_MAX_LENGTH (DEVICE_MSG_MAX_SIZE - 5)
 
-enum device_msg_type {
+enum sc_device_msg_type {
     DEVICE_MSG_TYPE_CLIPBOARD,
     DEVICE_MSG_TYPE_ACK_CLIPBOARD,
+    DEVICE_MSG_TYPE_UHID_OUTPUT,
 };
 
-struct device_msg {
-    enum device_msg_type type;
+struct sc_device_msg {
+    enum sc_device_msg_type type;
     union {
         struct {
             char *text; // owned, to be freed by free()
@@ -25,15 +26,20 @@ struct device_msg {
         struct {
             uint64_t sequence;
         } ack_clipboard;
+        struct {
+            uint16_t id;
+            uint16_t size;
+            uint8_t *data; // owned, to be freed by free()
+        } uhid_output;
     };
 };
 
 // return the number of bytes consumed (0 for no msg available, -1 on error)
 ssize_t
-device_msg_deserialize(const unsigned char *buf, size_t len,
-                       struct device_msg *msg);
+sc_device_msg_deserialize(const uint8_t *buf, size_t len,
+                          struct sc_device_msg *msg);
 
 void
-device_msg_destroy(struct device_msg *msg);
+sc_device_msg_destroy(struct sc_device_msg *msg);
 
 #endif
