@@ -4,11 +4,13 @@
 #include "common.h"
 
 #include <stdbool.h>
-#include <libavformat/avformat.h>
+#include <stdint.h>
+#include <libavutil/frame.h>
 #include <SDL2/SDL.h>
 
 #include "coords.h"
 #include "opengl.h"
+#include "options.h"
 
 #ifdef __APPLE__
 # define SC_DISPLAY_FORCE_OPENGL_CORE_PROFILE
@@ -20,7 +22,7 @@ struct sc_display {
 
     struct sc_opengl gl;
 #ifdef SC_DISPLAY_FORCE_OPENGL_CORE_PROFILE
-    SDL_GLContext *gl_context;
+    SDL_GLContext gl_context;
 #endif
 
     bool mipmaps;
@@ -32,6 +34,8 @@ struct sc_display {
         struct sc_size size;
         AVFrame *frame;
     } pending;
+
+    bool has_frame;
 };
 
 enum sc_display_result {
@@ -41,7 +45,8 @@ enum sc_display_result {
 };
 
 bool
-sc_display_init(struct sc_display *display, SDL_Window *window, bool mipmaps);
+sc_display_init(struct sc_display *display, SDL_Window *window,
+                SDL_Surface *icon_novideo, bool mipmaps);
 
 void
 sc_display_destroy(struct sc_display *display);
@@ -54,6 +59,6 @@ sc_display_update_texture(struct sc_display *display, const AVFrame *frame);
 
 enum sc_display_result
 sc_display_render(struct sc_display *display, const SDL_Rect *geometry,
-                  unsigned rotation);
+                  enum sc_orientation orientation);
 
 #endif
