@@ -9,7 +9,6 @@
 #include <SDL2/SDL_events.h>
 
 #include "coords.h"
-#include "options.h"
 
 /* The representation of input events in scrcpy is very close to the SDL API,
  * for simplicity.
@@ -394,6 +393,8 @@ struct sc_mouse_scroll_event {
     struct sc_position position;
     float hscroll;
     float vscroll;
+    int32_t hscroll_int;
+    int32_t vscroll_int;
     uint8_t buttons_state; // bitwise-OR of sc_mouse_button values
 };
 
@@ -412,18 +413,12 @@ struct sc_touch_event {
     float pressure;
 };
 
-enum sc_gamepad_device_event_type {
-    SC_GAMEPAD_DEVICE_ADDED,
-    SC_GAMEPAD_DEVICE_REMOVED,
-};
-
 // As documented in <https://wiki.libsdl.org/SDL2/SDL_JoystickID>:
 // The ID value starts at 0 and increments from there. The value -1 is an
 // invalid ID.
 #define SC_GAMEPAD_ID_INVALID UINT32_C(-1)
 
 struct sc_gamepad_device_event {
-    enum sc_gamepad_device_event_type type;
     uint32_t gamepad_id;
 };
 
@@ -503,16 +498,6 @@ sc_mouse_buttons_state_from_sdl(uint32_t buttons_state) {
     return buttons_state;
 }
 
-static inline enum sc_gamepad_device_event_type
-sc_gamepad_device_event_type_from_sdl_type(uint32_t type) {
-    assert(type == SDL_CONTROLLERDEVICEADDED
-        || type == SDL_CONTROLLERDEVICEREMOVED);
-    if (type == SDL_CONTROLLERDEVICEADDED) {
-        return SC_GAMEPAD_DEVICE_ADDED;
-    }
-    return SC_GAMEPAD_DEVICE_REMOVED;
-}
-
 static inline enum sc_gamepad_axis
 sc_gamepad_axis_from_sdl(uint8_t axis) {
     if (axis <= SDL_CONTROLLER_AXIS_TRIGGERRIGHT) {
