@@ -1,9 +1,7 @@
 #include "common.h"
 
-#include <assert.h>
 #include <stdbool.h>
-#include <unistd.h>
-#include <libavformat/avformat.h>
+#include <stdio.h>
 #ifdef HAVE_V4L2
 # include <libavdevice/avdevice.h>
 #endif
